@@ -36,6 +36,7 @@
 #include "gal/util/e-i18n.h"
 #include "gal/util/e-util.h"
 #include "gal/widgets/e-canvas.h"
+#include "gal/widgets/e-canvas-background.h"
 #include "gal/widgets/e-canvas-vbox.h"
 #include "gal/widgets/e-unicode.h"
 #include "e-table.h"
@@ -64,7 +65,7 @@
 #define e_table_item_leave_edit_(x) (e_table_item_leave_edit((x)))
 #endif
 
-static GtkObjectClass *e_table_parent_class;
+static GtkObjectClass *parent_class;
 
 enum {
 	CURSOR_CHANGE,
@@ -75,6 +76,8 @@ enum {
 	CLICK,
 	KEY_PRESS,
 	START_DRAG,
+	STATE_CHANGE,
+	WHITE_SPACE_EVENT,
 
 	TABLE_DRAG_BEGIN,
 	TABLE_DRAG_END,
@@ -93,7 +96,7 @@ enum {
 	ARG_0,
 	ARG_LENGTH_THRESHOLD,
 	ARG_MODEL,
-	ARG_UNIFORM_ROW_HEIGHT,
+	ARG_UNIFORM_ROW_HEIGHT
 };
 
 enum {
@@ -103,7 +106,7 @@ enum {
 	ET_SCROLL_RIGHT = 1 << 3
 };
 
-static gint et_signals [LAST_SIGNAL] = { 0, };
+static guint et_signals [LAST_SIGNAL] = { 0 };
 
 static void e_table_fill_table (ETable *e_table, ETableModel *model);
 static gboolean changed_idle (gpointer data);
@@ -185,6 +188,93 @@ et_disconnect_model (ETable *et)
 	et->table_rows_deleted_id = 0;
 }
 
+static void
+e_table_state_change (ETable *et)
+{
+	gtk_signal_emit (GTK_OBJECT (et),
+			 et_signals [STATE_CHANGE]);
+}
+
+#define CHECK_HORIZONTAL(et) if ((et)->horizontal_scrolling || (et)->horizontal_resize) e_table_header_update_horizontal (et->header);
+
+static void
+et_size_request (GtkWidget *widget, GtkRequisition *request)
+{
+	ETable *et = E_TABLE (widget);
+	if (GTK_WIDGET_CLASS (parent_class)->size_request)
+		GTK_WIDGET_CLASS (parent_class)->size_request (widget, request);
+	if (et->horizontal_resize)
+		request->width = MAX (request->width, et->header_width);
+}
+
+static void
+set_header_width (ETable *et)
+{
+	if (et->horizontal_resize) {
+		et->header_width = e_table_header_min_width (et->header);
+		gtk_widget_queue_resize (GTK_WIDGET (et));
+	}
+}
+
+static void
+structure_changed (ETableHeader *header, ETable *et)
+{
+	e_table_state_change (et);
+	set_header_width (et);
+}
+
+static void
+expansion_changed (ETableHeader *header, ETable *et)
+{
+	e_table_state_change (et);
+	set_header_width (et);
+}
+
+static void
+dimension_changed (ETableHeader *header, int total_width, ETable *et)
+{
+	set_header_width (et);
+}
+
+static void
+disconnect_header (ETable *e_table)
+{
+	if (e_table->header == NULL)
+		return;
+
+	if (e_table->structure_change_id)
+		gtk_signal_disconnect (GTK_OBJECT (e_table->header),
+				       e_table->structure_change_id);
+	if (e_table->expansion_change_id)
+		gtk_signal_disconnect (GTK_OBJECT (e_table->header),
+				       e_table->expansion_change_id);
+	if (e_table->dimension_change_id)
+		gtk_signal_disconnect (GTK_OBJECT (e_table->header),
+				       e_table->dimension_change_id);
+
+	gtk_object_unref(GTK_OBJECT(e_table->header));
+	e_table->header = NULL;
+}
+
+static void
+connect_header (ETable *e_table, ETableState *state)
+{
+	if (e_table->header != NULL)
+		disconnect_header (e_table);
+
+	e_table->header = e_table_state_to_header (GTK_WIDGET(e_table), e_table->full_header, state);
+
+	e_table->structure_change_id =
+		gtk_signal_connect (GTK_OBJECT (e_table->header), "structure_change",
+				    GTK_SIGNAL_FUNC (structure_changed), e_table);
+	e_table->expansion_change_id =
+		gtk_signal_connect (GTK_OBJECT (e_table->header), "expansion_change",
+				    GTK_SIGNAL_FUNC (expansion_changed), e_table);
+	e_table->dimension_change_id =
+		gtk_signal_connect (GTK_OBJECT (e_table->header), "dimension_change",
+				    GTK_SIGNAL_FUNC (dimension_changed), e_table);
+}
+
 static void
 et_destroy (GtkObject *object)
 {
@@ -192,62 +282,89 @@ et_destroy (GtkObject *object)
 
 	et_disconnect_model (et);
 
-	if (et->group_info_change_id)
+	if (et->search) {
+		if (et->search_search_id)
+			gtk_signal_disconnect (GTK_OBJECT (et->search),
+					       et->search_search_id);
+		if (et->search_accept_id)
+			gtk_signal_disconnect (GTK_OBJECT (et->search),
+					       et->search_accept_id);
+		gtk_object_unref (GTK_OBJECT (et->search));
+		et->search = NULL;
+	}
+
+	if (et->group_info_change_id) {
 		gtk_signal_disconnect (GTK_OBJECT (et->sort_info),
 				       et->group_info_change_id);
-	et->group_info_change_id = 0;
+		et->group_info_change_id = 0;
+	}
 	
-	if (et->reflow_idle_id)
+	if (et->sort_info_change_id) {
+		gtk_signal_disconnect (GTK_OBJECT (et->sort_info),
+				       et->sort_info_change_id);
+		et->sort_info_change_id = 0;
+	}
+
+	if (et->reflow_idle_id) {
 		g_source_remove(et->reflow_idle_id);
-	et->reflow_idle_id = 0;
+		et->reflow_idle_id = 0;
+	}
 
 	scroll_off (et);
 
-	if (et->model)
+	disconnect_header (et);
+
+	if (et->model) {
 		gtk_object_unref (GTK_OBJECT (et->model));
-	et->model = NULL;
+		et->model = NULL;
+	}
 
-	if (et->full_header)
+	if (et->full_header) {
 		gtk_object_unref (GTK_OBJECT (et->full_header));
-	et->full_header = NULL;
+		et->full_header = NULL;
+	}
 
-	if (et->header)
-		gtk_object_unref (GTK_OBJECT (et->header));
-	et->header = NULL;
-
-	if (et->sort_info)
+	if (et->sort_info) {
 		gtk_object_unref (GTK_OBJECT (et->sort_info));
-	et->sort_info = NULL;
+		et->sort_info = NULL;
+	}
 
-	if (et->sorter)
+	if (et->sorter) {
 		gtk_object_unref (GTK_OBJECT (et->sorter));
-	et->sorter = NULL;
+		et->sorter = NULL;
+	}
 
-	if (et->selection)
+	if (et->selection) {
 		gtk_object_unref (GTK_OBJECT (et->selection));
-	et->selection = NULL;
+		et->selection = NULL;
+	}
 
-	if (et->spec)
+	if (et->spec) {
 		gtk_object_unref (GTK_OBJECT (et->spec));
-	et->spec = NULL;
+		et->spec = NULL;
+	}
 
-	if (et->header_canvas != NULL)
+	if (et->header_canvas != NULL) {
 		gtk_widget_destroy (GTK_WIDGET (et->header_canvas));
-	et->header_canvas = NULL;
+		et->header_canvas = NULL;
+	}
 
-	if (et->site != NULL)
+	if (et->site != NULL) {
 		e_table_drag_source_unset (et);
-	et->site = NULL;
+		et->site = NULL;
+	}
 
-	if (et->table_canvas != NULL)
+	if (et->table_canvas != NULL) {
 		gtk_widget_destroy (GTK_WIDGET (et->table_canvas));
-	et->table_canvas = NULL;
+		et->table_canvas = NULL;
+	}
 
-	if (et->rebuild_idle_id != 0)
+	if (et->rebuild_idle_id != 0) {
 		g_source_remove (et->rebuild_idle_id);
-	et->rebuild_idle_id = 0;
+		et->rebuild_idle_id = 0;
+	}
 
-	(*e_table_parent_class->destroy)(object);
+	parent_class->destroy (object);
 }
 
 static void
@@ -256,8 +373,94 @@ et_finalize (GObject *object)
 	ETable *et = E_TABLE (object);
 
 	g_free (et->click_to_add_message);
+	et->click_to_add_message = NULL;
 
-	G_OBJECT_CLASS (e_table_parent_class)->finalize (object);
+	g_free(et->domain);
+	et->domain = NULL;
+
+	G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
+static void
+et_unrealize (GtkWidget *widget)
+{
+	scroll_off (E_TABLE (widget));
+
+	if (GTK_WIDGET_CLASS (parent_class)->unrealize)
+		GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
+}
+
+static gboolean
+check_row (ETable *et, int model_row, int col, ETableSearchFunc search, char *string)
+{
+	const void *value;
+
+	value = e_table_model_value_at (et->model, col, model_row);
+
+	return search (value, string);
+}
+
+static gboolean
+et_search_search (ETableSearch *search, char *string, ETableSearchFlags flags, ETable *et)
+{
+	int cursor;
+	int rows;
+	int i;
+	int col;
+	ETableSearchFunc search_func;
+
+	col = et->current_search_col;
+	if (col == -1)
+		return FALSE;
+
+	rows = e_table_model_row_count (et->model);
+
+	search_func = et->current_search;
+
+	gtk_object_get(GTK_OBJECT(et->selection),
+		       "cursor_row", &cursor,
+		       NULL);
+
+	if ((flags & E_TABLE_SEARCH_FLAGS_CHECK_CURSOR_FIRST) && cursor < rows && cursor >= 0 && check_row (et, cursor, col, search_func, string))
+		return TRUE;
+
+	cursor = e_sorter_model_to_sorted (E_SORTER (et->sorter), cursor);
+
+	for (i = cursor + 1; i < rows; i++) {
+		int model_row = e_sorter_sorted_to_model (E_SORTER (et->sorter), i);
+		if (check_row (et, model_row, col, search_func, string)) {
+			e_selection_model_select_as_key_press(E_SELECTION_MODEL (et->selection), model_row, col, GDK_CONTROL_MASK);
+			return TRUE;
+		}
+	}
+
+	for (i = 0; i < cursor; i++) {
+		int model_row = e_sorter_sorted_to_model (E_SORTER (et->sorter), i);
+		if (check_row (et, model_row, col, search_func, string)) {
+			e_selection_model_select_as_key_press(E_SELECTION_MODEL (et->selection), model_row, col, GDK_CONTROL_MASK);
+			return TRUE;
+		}
+	}
+
+	cursor = e_sorter_sorted_to_model (E_SORTER (et->sorter), cursor);
+
+	/* Check if the cursor row is the only matching row. */
+	return (!(flags & E_TABLE_SEARCH_FLAGS_CHECK_CURSOR_FIRST) && cursor < rows && cursor >= 0 && check_row (et, cursor, col, search_func, string));
+}
+
+static void
+et_search_accept (ETableSearch *search, ETable *et)
+{
+	int col, cursor;
+
+	col = et->current_search_col;
+	if (col == -1)
+		return;
+
+	gtk_object_get(GTK_OBJECT(et->selection),
+		       "cursor_row", &cursor,
+		       NULL);
+	e_selection_model_select_as_key_press(E_SELECTION_MODEL (et->selection), cursor, col, 0);
 }
 
 static void
@@ -268,42 +471,62 @@ e_table_init (GtkObject *object)
 
 	GTK_WIDGET_SET_FLAGS (e_table, GTK_CAN_FOCUS);
 
-	gtk_table->homogeneous                      = FALSE;
+	gtk_table->homogeneous          = FALSE;
 
-	e_table->sort_info                          = NULL;
-	e_table->group_info_change_id               = 0;
-	e_table->reflow_idle_id                     = 0;
-	e_table->scroll_idle_id               = 0;
+	e_table->sort_info              = NULL;
+	e_table->group_info_change_id   = 0;
+	e_table->sort_info_change_id    = 0;
+	e_table->structure_change_id    = 0;
+	e_table->expansion_change_id    = 0;
+	e_table->dimension_change_id    = 0;
+	e_table->reflow_idle_id         = 0;
+	e_table->scroll_idle_id         = 0;
 
-	e_table->alternating_row_colors             = 1;
-	e_table->horizontal_draw_grid               = 1;
-	e_table->vertical_draw_grid                 = 1;
-	e_table->draw_focus                         = 1;
-	e_table->cursor_mode                        = E_CURSOR_SIMPLE;
-	e_table->length_threshold                   = 200;
-	e_table->uniform_row_height                 = FALSE;
+	e_table->alternating_row_colors = 1;
+	e_table->horizontal_draw_grid   = 1;
+	e_table->vertical_draw_grid     = 1;
+	e_table->draw_focus             = 1;
+	e_table->cursor_mode            = E_CURSOR_SIMPLE;
+	e_table->length_threshold       = 200;
+	e_table->uniform_row_height     = FALSE;
 
-	e_table->need_rebuild                       = 0;
-	e_table->rebuild_idle_id                    = 0;
+	e_table->need_rebuild           = 0;
+	e_table->rebuild_idle_id        = 0;
 
-	e_table->horizontal_scrolling               = FALSE;
+	e_table->horizontal_scrolling   = FALSE;
+	e_table->horizontal_resize      = FALSE;
 
-	e_table->click_to_add_message               = NULL;
+	e_table->click_to_add_message   = NULL;
+	e_table->domain                 = NULL;
 
-	e_table->drag_get_data_row                  = -1;
-	e_table->drag_get_data_col                  = -1;
-	e_table->drop_row                           = -1;
-	e_table->drop_col                           = -1;
-	e_table->site                               = NULL;
+	e_table->drag_get_data_row      = -1;
+	e_table->drag_get_data_col      = -1;
+	e_table->drop_row               = -1;
+	e_table->drop_col               = -1;
+	e_table->site                   = NULL;
 
-	e_table->do_drag                            = 0;
+	e_table->do_drag                = 0;
 
-	e_table->sorter                             = NULL;
-	e_table->selection                          = e_table_selection_model_new();
+	e_table->sorter                 = NULL;
+	e_table->selection              = e_table_selection_model_new();
 	gtk_object_ref (GTK_OBJECT (e_table->selection));
 	gtk_object_sink (GTK_OBJECT (e_table->selection));
-	e_table->cursor_loc                         = E_TABLE_CURSOR_LOC_NONE;
-	e_table->spec                               = NULL;
+	e_table->cursor_loc             = E_TABLE_CURSOR_LOC_NONE;
+	e_table->spec                   = NULL;
+
+	e_table->search                 = e_table_search_new();
+
+	e_table->search_search_id       = 
+		gtk_signal_connect (GTK_OBJECT (e_table->search), "search",
+				    GTK_SIGNAL_FUNC (et_search_search), e_table);
+	e_table->search_accept_id       = 
+		gtk_signal_connect (GTK_OBJECT (e_table->search), "accept",
+				    GTK_SIGNAL_FUNC (et_search_accept), e_table);
+
+	e_table->current_search         = NULL;
+	e_table->current_search_col     = -1;
+
+	e_table->header_width           = 0;
 }
 
 /* Grab_focus handler for the ETable */
@@ -366,12 +589,12 @@ header_canvas_size_allocate (GtkWidget *widget, GtkAllocation *alloc, ETable *e_
 	   header is correct */
 	if (GTK_WIDGET (e_table->header_canvas)->allocation.height !=
 	    E_TABLE_HEADER_ITEM (e_table->header_item)->height)
-		gtk_widget_set_usize (GTK_WIDGET (e_table->header_canvas), -1,
+		gtk_widget_set_usize (GTK_WIDGET (e_table->header_canvas), -2,
 				      E_TABLE_HEADER_ITEM (e_table->header_item)->height);
 }
 
 static void
-sort_info_changed (ETableSortInfo *info, ETable *et)
+group_info_changed (ETableSortInfo *info, ETable *et)
 {
 	gboolean will_be_grouped = e_table_sort_info_grouping_get_count(info) > 0;
 	if (et->is_grouped || will_be_grouped) {
@@ -382,6 +605,13 @@ sort_info_changed (ETableSortInfo *info, ETable *et)
 			et->rebuild_idle_id = g_idle_add_full (20, changed_idle, et, NULL);
 		}
 	}
+	e_table_state_change (et);
+}
+
+static void
+sort_info_changed (ETableSortInfo *info, ETable *et)
+{
+	e_table_state_change (et);
 }
 
 static void
@@ -411,7 +641,7 @@ e_table_setup_header (ETable *e_table)
 		GTK_OBJECT (e_table->header_canvas), "size_allocate",
 		GTK_SIGNAL_FUNC (header_canvas_size_allocate), e_table);
 
-	gtk_widget_set_usize (GTK_WIDGET (e_table->header_canvas), -1,
+	gtk_widget_set_usize (GTK_WIDGET (e_table->header_canvas), -2,
 			      E_TABLE_HEADER_ITEM (e_table->header_item)->height);
 }
 
@@ -440,11 +670,6 @@ table_canvas_reflow_idle (ETable *e_table)
 						0, 0, width - 1, height - 1);
 		set_header_canvas_width (e_table);
 	}
-	gtk_object_set (GTK_OBJECT (e_table->white_item),
-			"y1", item_height,
-			"x2", width,
-			"y2", height,
-			NULL);
 	e_table->reflow_idle_id = 0;
 	return FALSE;
 }
@@ -470,11 +695,6 @@ table_canvas_size_allocate (GtkWidget *widget, GtkAllocation *alloc,
 	gtk_object_set (GTK_OBJECT (e_table->header),
 			"width", width,
 			NULL);
-	gtk_object_set (GTK_OBJECT (e_table->white_item),
-			"y1", item_height + 1,
-			"x2", width,
-			"y2", height,
-			NULL);
 	if (e_table->reflow_idle_id)
 		g_source_remove(e_table->reflow_idle_id);
 	table_canvas_reflow_idle(e_table);
@@ -587,7 +807,17 @@ group_key_press (ETableGroup *etg, int row, int col, GdkEvent *event, ETable *et
 		e_selection_model_select_as_key_press (E_SELECTION_MODEL (et->selection), row_local, col_local, key->state);
 		return_val = 1;
 		break;
+	case GDK_BackSpace:
+		if (e_table_search_backspace (et->search))
+			return TRUE;
+		/* Fall through */
 	default:
+		if ((key->state & ~(GDK_SHIFT_MASK | GDK_LOCK_MASK)) == 0
+		    && ((key->keyval >= GDK_a && key->keyval <= GDK_z) ||
+			(key->keyval >= GDK_A && key->keyval <= GDK_Z) ||
+			(key->keyval >= GDK_0 && key->keyval <= GDK_9))) {
+			e_table_search_input_character (et->search, key->keyval);
+		}
 		gtk_signal_emit (GTK_OBJECT (et),
 				 et_signals [KEY_PRESS],
 				 row, col, event, &return_val);
@@ -623,8 +853,7 @@ et_table_row_changed (ETableModel *table_model, int row, ETable *et)
 	if (!et->need_rebuild) {
 		if (e_table_group_remove (et->group, row))
 			e_table_group_add (et->group, row);
-		if (et->horizontal_scrolling)
-			e_table_header_update_horizontal(et->header);
+		CHECK_HORIZONTAL(et);
 	}
 }
 
@@ -645,8 +874,7 @@ et_table_rows_inserted (ETableModel *table_model, int row, int count, ETable *et
 			e_table_group_increment(et->group, row, count);
 		for (i = 0; i < count; i++)
 			e_table_group_add (et->group, row + i);
-		if (et->horizontal_scrolling)
-			e_table_header_update_horizontal(et->header);
+		CHECK_HORIZONTAL(et);
 	}
 }
 
@@ -660,8 +888,7 @@ et_table_rows_deleted (ETableModel *table_model, int row, int count, ETable *et)
 			e_table_group_remove (et->group, row + i);
 		if (row != row_count)
 			e_table_group_decrement(et->group, row, count);
-		if (et->horizontal_scrolling)
-			e_table_header_update_horizontal(et->header);
+		CHECK_HORIZONTAL(et);
 	}
 }
 
@@ -756,8 +983,7 @@ changed_idle (gpointer data)
 	et->need_rebuild = 0;
 	et->rebuild_idle_id = 0;
 
-	if (et->horizontal_scrolling)
-		e_table_header_update_horizontal(et->header);
+	CHECK_HORIZONTAL(et);
 
 	return FALSE;
 }
@@ -769,6 +995,19 @@ et_canvas_realize (GtkWidget *canvas, ETable *e_table)
 		e_table->white_item,
 		"fill_color_gdk", &GTK_WIDGET(e_table->table_canvas)->style->base[GTK_STATE_NORMAL],
 		NULL);
+
+	CHECK_HORIZONTAL(e_table);
+	set_header_width (e_table);
+}
+
+static gint
+white_item_event (GnomeCanvasItem *white_item, GdkEvent *event, ETable *e_table)
+{
+	int return_val = 0;
+	gtk_signal_emit (GTK_OBJECT (e_table),
+			 et_signals [WHITE_SPACE_EVENT],
+			 event, &return_val);
+	return return_val;
 }
 
 static void
@@ -897,20 +1136,19 @@ e_table_setup_table (ETable *e_table, ETableHeader *full_header, ETableHeader *h
 
 	e_table->white_item = gnome_canvas_item_new(
 		gnome_canvas_root(e_table->table_canvas),
-		gnome_canvas_rect_get_type(),
-		"x1", (double) 0,
-		"y1", (double) 0,
-		"x2", (double) 100,
-		"y2", (double) 100,
+		e_canvas_background_get_type(),
 		"fill_color_gdk", &GTK_WIDGET(e_table->table_canvas)->style->base[GTK_STATE_NORMAL],
 		NULL);
 
-	gtk_signal_connect (
-		GTK_OBJECT(e_table->table_canvas), "realize",
-		GTK_SIGNAL_FUNC(et_canvas_realize), e_table);
-	gtk_signal_connect (
-		GTK_OBJECT(gnome_canvas_root (e_table->table_canvas)), "event",
-		GTK_SIGNAL_FUNC(et_canvas_root_event), e_table);
+	gtk_signal_connect (GTK_OBJECT (e_table->white_item), "event",
+			    GTK_SIGNAL_FUNC (white_item_event), e_table);
+
+	gtk_signal_connect (GTK_OBJECT(e_table->table_canvas), "realize",
+			    GTK_SIGNAL_FUNC(et_canvas_realize), e_table);
+
+	gtk_signal_connect (GTK_OBJECT(gnome_canvas_root (e_table->table_canvas)), "event",
+			    GTK_SIGNAL_FUNC(et_canvas_root_event), e_table);
+
 	e_table->canvas_vbox = gnome_canvas_item_new(
 		gnome_canvas_root(e_table->table_canvas),
 		e_canvas_vbox_get_type(),
@@ -961,10 +1199,7 @@ e_table_fill_table (ETable *e_table, ETableModel *model)
 void
 e_table_set_state_object(ETable *e_table, ETableState *state)
 {
-	if (e_table->header)
-		gtk_object_unref(GTK_OBJECT(e_table->header));
-	e_table->header = e_table_state_to_header (GTK_WIDGET(e_table), e_table->full_header, state);
-
+	connect_header (e_table, state);
 	gtk_object_set (GTK_OBJECT (e_table->header),
 			"width", (double) (GTK_WIDGET(e_table->table_canvas)->allocation.width),
 			NULL);
@@ -973,6 +1208,9 @@ e_table_set_state_object(ETable *e_table, ETableState *state)
 		if (e_table->group_info_change_id)
 			gtk_signal_disconnect (GTK_OBJECT (e_table->sort_info),
 					       e_table->group_info_change_id);
+		if (e_table->sort_info_change_id)
+			gtk_signal_disconnect (GTK_OBJECT (e_table->sort_info),
+					       e_table->sort_info_change_id);
 		gtk_object_unref(GTK_OBJECT(e_table->sort_info));
 	}
 	if (state->sort_info) {
@@ -981,6 +1219,11 @@ e_table_set_state_object(ETable *e_table, ETableState *state)
 		e_table->group_info_change_id =
 			gtk_signal_connect (GTK_OBJECT (e_table->sort_info),
 					    "group_info_changed",
+					    GTK_SIGNAL_FUNC (group_info_changed),
+					    e_table);
+		e_table->sort_info_change_id =
+			gtk_signal_connect (GTK_OBJECT (e_table->sort_info),
+					    "sort_info_changed",
 					    GTK_SIGNAL_FUNC (sort_info_changed),
 					    e_table);
 	}
@@ -1161,6 +1404,7 @@ et_real_construct (ETable *e_table, ETableModel *etm, ETableExtras *ete,
 		   ETableSpecification *specification, ETableState *state)
 {
 	int row = 0;
+	int col_count, i;
 
 	if (ete)
 		gtk_object_ref(GTK_OBJECT(ete));
@@ -1170,9 +1414,11 @@ et_real_construct (ETable *e_table, ETableModel *etm, ETableExtras *ete,
 		gtk_object_sink(GTK_OBJECT(ete));
 	}
 
+	e_table->domain = g_strdup (specification->domain);
+
 	e_table->use_click_to_add = specification->click_to_add;
 	e_table->use_click_to_add_end = specification->click_to_add_end;
-	e_table->click_to_add_message = e_utf8_from_locale_string (gettext (specification->click_to_add_message));
+	e_table->click_to_add_message = e_utf8_from_locale_string (dgettext (e_table->domain, specification->click_to_add_message));
 	e_table->alternating_row_colors = specification->alternating_row_colors;
 	e_table->horizontal_draw_grid = specification->horizontal_draw_grid;
 	e_table->vertical_draw_grid = specification->vertical_draw_grid;
@@ -1182,6 +1428,16 @@ et_real_construct (ETable *e_table, ETableModel *etm, ETableExtras *ete,
 	gtk_object_ref (GTK_OBJECT (e_table->full_header));
 	gtk_object_sink (GTK_OBJECT (e_table->full_header));
 
+	col_count = e_table_header_count (e_table->full_header);
+	for (i = 0; i < col_count; i++) {
+		ETableCol *col = e_table_header_get_column(e_table->full_header, i);
+		if (col && col->search) {
+			e_table->current_search_col = col->col_idx;
+			e_table->current_search = col->search;
+			break;
+		}
+	}
+
 	gtk_object_set(GTK_OBJECT(e_table->selection),
 		       "selection_mode", specification->selection_mode,
 		       "cursor_mode", specification->cursor_mode,
@@ -1192,10 +1448,9 @@ et_real_construct (ETable *e_table, ETableModel *etm, ETableExtras *ete,
 
 	gtk_widget_push_colormap (gdk_rgb_get_cmap ());
 
-	e_table->header = e_table_state_to_header (GTK_WIDGET(e_table), e_table->full_header, state);
-	gtk_object_ref (GTK_OBJECT (e_table->header));
-	gtk_object_sink (GTK_OBJECT (e_table->header));
+	connect_header (e_table, state);
 	e_table->horizontal_scrolling = specification->horizontal_scrolling;
+	e_table->horizontal_resize = specification->horizontal_resize;
 	e_table->allow_grouping = specification->allow_grouping;
 
 	e_table->sort_info = state->sort_info;
@@ -1204,6 +1459,10 @@ et_real_construct (ETable *e_table, ETableModel *etm, ETableExtras *ete,
 
 	e_table->group_info_change_id =
 		gtk_signal_connect (GTK_OBJECT (e_table->sort_info), "group_info_changed",
+				    GTK_SIGNAL_FUNC (group_info_changed), e_table);
+
+	e_table->sort_info_change_id =
+		gtk_signal_connect (GTK_OBJECT (e_table->sort_info), "sort_info_changed",
 				    GTK_SIGNAL_FUNC (sort_info_changed), e_table);
 
 
@@ -2536,7 +2795,9 @@ context_destroyed (gpointer data)
 {
 	ETable *et = data;
 	/* if (!GTK_OBJECT_DESTROYED (et)) */
+#ifndef NO_WARNINGS
 #warning FIXME
+#endif
 	{
 		et->last_drop_x       = 0;
 		et->last_drop_y       = 0;
@@ -2706,11 +2967,11 @@ e_table_class_init (ETableClass *class)
 	GtkWidgetClass *widget_class;
 	GtkContainerClass *container_class;
 
-	object_class = (GtkObjectClass *) class;
-	widget_class = (GtkWidgetClass *) class;
-	container_class = (GtkContainerClass *) class;
+	object_class                    = (GtkObjectClass *) class;
+	widget_class                    = (GtkWidgetClass *) class;
+	container_class                 = (GtkContainerClass *) class;
 
-	e_table_parent_class = gtk_type_class (PARENT_TYPE);
+	parent_class                    = gtk_type_class (PARENT_TYPE);
 
 	object_class->destroy           = et_destroy;
 	G_OBJECT_CLASS (object_class)->finalize = et_finalize;
@@ -2718,6 +2979,8 @@ e_table_class_init (ETableClass *class)
 	object_class->get_arg           = et_get_arg;
 
 	widget_class->grab_focus        = et_grab_focus;
+	widget_class->unrealize         = et_unrealize;
+	widget_class->size_request      = et_size_request;
 
 	widget_class->focus             = et_focus;
 
@@ -2729,6 +2992,8 @@ e_table_class_init (ETableClass *class)
 	class->click                    = NULL;
 	class->key_press                = NULL;
 	class->start_drag               = et_real_start_drag;
+	class->state_change             = NULL;
+	class->white_space_event        = NULL;
 
 	class->table_drag_begin         = NULL;
 	class->table_drag_end           = NULL;
@@ -2809,6 +3074,22 @@ e_table_class_init (ETableClass *class)
 				GTK_TYPE_INT, 3, GTK_TYPE_INT,
 				GTK_TYPE_INT, GDK_TYPE_EVENT);
 
+	et_signals [STATE_CHANGE] =
+		gtk_signal_new ("state_change",
+				GTK_RUN_LAST,
+				E_OBJECT_CLASS_TYPE (object_class),
+				GTK_SIGNAL_OFFSET (ETableClass, state_change),
+				gtk_marshal_NONE__NONE,
+				GTK_TYPE_NONE, 0);
+
+	et_signals [WHITE_SPACE_EVENT] =
+		gtk_signal_new ("white_space_event",
+				GTK_RUN_LAST,
+				E_OBJECT_CLASS_TYPE (object_class),
+				GTK_SIGNAL_OFFSET (ETableClass, white_space_event),
+				gtk_marshal_INT__POINTER,
+				GTK_TYPE_INT, 1, GDK_TYPE_EVENT);
+
 	et_signals[TABLE_DRAG_BEGIN] =
 		gtk_signal_new ("table_drag_begin",
 				GTK_RUN_LAST,
@@ -2926,4 +3207,4 @@ e_table_class_init (ETableClass *class)
 				 GTK_ARG_READABLE, ARG_MODEL);
 }
 
-E_MAKE_TYPE(e_table, "ETable", ETable, e_table_class_init, e_table_init, PARENT_TYPE);
+E_MAKE_TYPE(e_table, "ETable", ETable, e_table_class_init, e_table_init, PARENT_TYPE)
